﻿using Marten;

namespace Brizco.Infrastructure.Marten;

public class MartenRepository<TMartenEntity>(IDocumentStore documentStore) : IMartenRepository<TMartenEntity>
    where TMartenEntity : IMartenEntity
{
    public async Task<List<TMartenEntity>> GetEntitiesAsync(CancellationToken cancellation)
    {
        await using var session = documentStore.QuerySession();
        var entities = await session.Query<TMartenEntity>().ToListAsync(cancellation);
        return entities.ToList();
    }

    public async Task<List<TMartenEntity>> GetEntitiesAsync(Expression<Func<TMartenEntity, bool>> expression, CancellationToken cancellation)
    {
        await using var session = documentStore.QuerySession();
        var entities = await session.Query<TMartenEntity>().Where(expression).ToListAsync(cancellation);
        return entities.ToList();
    }
    public async Task<IQueryable<TMartenEntity>> GetQueryAsync(Expression<Func<TMartenEntity, bool>> expression)
    {
        await using var session = documentStore.QuerySession();
        var entities = session.Query<TMartenEntity>().Where(expression);
        return entities;
    }

    public async Task<TMartenEntity> GetEntityAsync(Guid id, CancellationToken cancellation)
    {
        await using var session = documentStore.QuerySession();
        var setting = await session.LoadAsync<TMartenEntity>(id, cancellation);
        if (setting == null)
            throw new AppException($"{nameof(TMartenEntity)} not found", ApiResultStatusCode.NotFound);
        return setting;
    }

    public async Task<TMartenEntity?> GetEntityAsync(Expression<Func<TMartenEntity, bool>> expression, CancellationToken cancellation)
    {
        await using var session = documentStore.QuerySession();
        var entity = await session.Query<TMartenEntity>().FirstOrDefaultAsync(expression, cancellation);
        return entity;
    }

    public async Task AddOrUpdateEntityAsync(TMartenEntity entity, CancellationToken cancellation)
    {
        if (entity == null)
            throw new AppException($"{nameof(entity)} is null", ApiResultStatusCode.BadRequest);

        await using var session = documentStore.LightweightSession();
        session.Store(entity);
        await session.SaveChangesAsync(cancellation);
    }

    public async Task RemoveEntityAsync(TMartenEntity entity, CancellationToken cancellation)
    {
        if (entity == null)
            throw new AppException($"{nameof(entity)} is null", ApiResultStatusCode.BadRequest);
        await using var session = documentStore.LightweightSession();
        session.Delete(entity);
        await session.SaveChangesAsync(cancellation);
    }
}